@@ -5,11 +5,10 @@
 
 #include <time.h>
 
-#define CONF_FILE "conf.lua" /**< Configuration file by default. */
-
 /**
  * CONFIGURATION DEFAULTS
  */
+#define NUM_BACKUPS_DEFAULT 5 /**< Number of backups. */
 /* Gameplay options */
 #define DOUBLETAP_SENSITIVITY_DEFAULT                                          \
    250 /**< Default afterburner sensitivity. */
@@ -40,9 +39,9 @@
    720 /**< Minimum screen height (below which graphics are downscaled). */
 #define RESOLUTION_W_DEFAULT RESOLUTION_W_MIN /**< Default screen width. */
 #define RESOLUTION_H_DEFAULT RESOLUTION_H_MIN /**< Default screen height. */
-#define FULLSCREEN_DEFAULT 0 /**< Whether to run in fullscreen mode. */
-#define FULLSCREEN_MODESETTING                                                 \
-   0                    /**< Whether fullscreen uses video modesetting. */
+#define NOT_RESIZABLE_DEFAULT 0 /**< Whether or not the game is resizable. */
+#define FULLSCREEN_DEFAULT 0    /**< Whether to run in fullscreen mode. */
+#define DISABLE_SCREEN_SHAKE_DEFAULT 0
 #define FSAA_DEFAULT 1  /**< Whether to use Full Screen Anti-Aliasing. */
 #define VSYNC_DEFAULT 0 /**< Whether to wait for vertical sync. */
 #define SCALE_FACTOR_DEFAULT 1. /**< Default scale factor. */
@@ -61,8 +60,10 @@
    0. /**< Intensity of the colourblindness correction. */
 #define GAME_SPEED_DEFAULT 1. /**< Game speed. */
 #define HEALTHBARS_DEFAULT 1  /**< Whether or not to show pilot health bars. */
+#define FPS_SHOW_DEFAULT 0
+#define PAUSE_SHOW_DEFAULT 1
 #define BG_BRIGHTNESS_DEFAULT                                                  \
-   0.5 /**< How much to darken (or lighten) the backgrounds. */
+   0.7 /**< How much to darken (or lighten) the backgrounds. */
 #define NEBU_NONUNIFORMITY_DEFAULT                                             \
    1. /**< How much to darken (or lighten) the nebula stuff. */
 #define NEBU_SATURATION_DEFAULT                                                \
@@ -79,6 +80,18 @@
 #define FONT_SIZE_SMALL_DEFAULT 11   /**< Default small font size. */
 #define LOW_MEMORY_DEFAULT 0         /**< Default for low memory mode. */
 #define MAX_3D_TEX_SIZE 256          /**< Maximum 3D texture size. */
+#define ALWAYS_RADAR_DEFAULT 0
+#define SHOW_VIEWPORT_DEFAULT 0
+#define DEVMODE_DEFAULT 0
+#define DEVAUTOSAVE_DEFAULT 0
+#define LUA_ENET_DEFAULT 0
+#define LUA_REPL_DEFAULT 0
+#define TRANSLATION_WARNING_SEEN_DEFAULT 0
+#define FPU_EXCEPT_DEFAULT 0
+#define MESSAGE_VISIBLE_DEFAULT 5
+#define REPEAT_DELAY_DEFAULT 500
+#define REPEAT_FREQ_DEFAULT 30
+#define CONF_NOSAVE_DEFAULT 0
 /* Audio options */
 #define USE_EFX_DEFAULT 1 /**< Whether or not to use EFX (if using OpenAL). */
 #define MUTE_SOUND_DEFAULT 0      /**< Whether sound should be disabled. */
@@ -99,25 +112,25 @@ typedef struct PlayerConf_s {
    char *ndata;    /**< Ndata path to use. */
    char *datapath; /**< Path for user data (saves, screenshots, etc.). */
 
+   /* Saves. */
+   int num_backups; /**< Number of backups. */
+
    /* Language. */
    char *language; /**< Language to use. */
 
    /* OpenGL properties. */
-   int fsaa;  /**< Full Scene Anti-Aliasing to use. */
-   int vsync; /**< Whether or not to use vsync. */
+   unsigned char fsaa;  /**< Full Scene Anti-Aliasing to use. */
+   int           vsync; /**< Whether or not to use vsync. */
 
    /* Video options. */
-   int    width;           /**< Width of the window to use. */
-   int    height;          /**< Height of the window to use. */
-   int    explicit_dim;    /**< Dimension is explicit. */
-   double scalefactor;     /**< Amount to reduce resolution by. */
-   double nebu_scale;      /**< Downscaling factor for the expensively rendered
-                              nebula. */
-   int    fullscreen;      /**< Whether or not game is fullscreen. */
-   int    modesetting;     /**< Whether to use modesetting for fullscreen. */
-   int    notresizable;    /**< Whether or not the window is resizable. */
-   int    borderless;      /**< Whether to disable window decorations. */
-   int    minimize;        /**< Whether to minimize on focus loss. */
+   unsigned int width;       /**< Width of the window to use. */
+   unsigned int height;      /**< Height of the window to use. */
+   double       scalefactor; /**< Amount to reduce resolution by. */
+   double       nebu_scale; /**< Downscaling factor for the expensively rendered
+                               nebula. */
+   int    fullscreen;       /**< Whether or not game is fullscreen. */
+   int    notresizable;     /**< Whether or not the window is resizable. */
+   int    minimize;         /**< Whether to minimize on focus loss. */
    double colourblind_sim; /**< Whether to enable colourblindness simulation. */
    int    colourblind_type; /**< Type of colourblindness. */
    double
@@ -158,6 +171,7 @@ typedef struct PlayerConf_s {
    double map_overlay_opacity; /**< Map overlay opacity. */
    int    big_icons;           /**< Use big icons or not. */
    int    always_radar;        /**< Radar is always visible. */
+   int    show_viewport;       /**< Show viewport in the radar/overlay map. */
 
    /* Keyrepeat. */
    unsigned int repeat_delay; /**< Time in ms before start repeating. */
@@ -207,6 +221,9 @@ typedef struct PlayerConf_s {
 
    /* Editor. */
    char *dev_data_dir; /**< Path where most data should be. */
+
+   // Special
+   int exit_main_menu;
 } PlayerConf_t;
 extern PlayerConf_t conf; /**< Player configuration. */
 
