@@ -11,10 +11,9 @@
  *  to and open an entire set of events.
  */
 /** @cond */
-#include "nstring.h"
 #include <stdlib.h>
 
-#include "SDL_timer.h"
+#include <SDL3/SDL_timer.h>
 
 #include "naev.h"
 /** @endcond */
@@ -33,6 +32,7 @@
 #include "nlua_camera.h"
 #include "nlua_evt.h"
 #include "nlua_hook.h"
+#include "nlua_munition.h"
 #include "nlua_music.h"
 #include "nlua_tex.h"
 #include "nlua_tk.h"
@@ -113,6 +113,19 @@ Event_t *event_get( unsigned int eventid )
    return NULL;
 }
 
+int event_exists( unsigned int eventid )
+{
+   return event_get( eventid ) != NULL;
+}
+
+nlua_env *event_getEnv( unsigned int eventid )
+{
+   Event_t *ev = event_get( eventid );
+   if ( ev == NULL )
+      return NULL;
+   return ev->env;
+}
+
 /**
  * @brief Starts an event.
  *
@@ -140,7 +153,7 @@ int event_start( const char *name, unsigned int *id )
  * @brief Gets the name of the event data.
  *
  *    @param eventid Event to get name of data from.
- *    @return Name of data ev has.
+ *    @return Name of data.
  */
 const char *event_getData( unsigned int eventid )
 {
@@ -164,6 +177,17 @@ int event_isUnique( unsigned int eventid )
    return !!( event_data[ev->data].flags & EVENT_FLAG_UNIQUE );
 }
 
+/**
+ * @brief Gets the priority of an event.
+ */
+int event_priority( unsigned int eventid )
+{
+   const Event_t *ev = event_get( eventid );
+   if ( ev == NULL )
+      return 5;
+   return event_data[ev->data].priority;
+}
+
 /**
  * @brief Generates a new event ID.
  */
@@ -215,6 +239,7 @@ static int event_create( int dataid, unsigned int *id )
    nlua_loadBackground( ev->env );
    nlua_loadMusic( ev->env );
    nlua_loadTk( ev->env );
+   nlua_loadMunition( ev->env );
 
    /* Create the "mem" table for persistence. */
    lua_newtable( naevL );
@@ -487,7 +512,7 @@ static int event_parseXML( EventData *temp, const xmlNodePtr parent )
       /* Tags. */
       if ( xml_isNode( node, "tags" ) ) {
          xmlNodePtr cur = node->children;
-         temp->tags     = array_create( char     *);
+         temp->tags     = array_create( char * );
          do {
             xml_onlyNodes( cur );
             if ( xml_isNode( cur, "tag" ) ) {
@@ -574,7 +599,8 @@ int events_load( void )
    /* Run over events. */
    event_data = array_create_size( EventData, array_size( event_files ) );
    for ( int i = 0; i < array_size( event_files ); i++ ) {
-      event_parseFile( event_files[i], NULL );
+      if ( ndata_matchExt( event_files[i], "lua" ) )
+         event_parseFile( event_files[i], NULL );
       free( event_files[i] );
    }
    array_free( event_files );
@@ -636,9 +662,9 @@ static int event_parseFile( const char *file, EventData *temp )
    }
 
    /* Skip if no XML. */
-   pos = strnstr( filebuf, "</event>", bufsize );
+   pos = SDL_strnstr( filebuf, "</event>", bufsize );
    if ( pos == NULL ) {
-      pos = strnstr( filebuf, "function create", bufsize );
+      pos = SDL_strnstr( filebuf, "function create", bufsize );
       if ( ( pos != NULL ) && !strncmp( pos, "--common", bufsize ) )
          WARN( _( "Event '%s' has create function but no XML header!" ), file );
       free( filebuf );
@@ -646,8 +672,8 @@ static int event_parseFile( const char *file, EventData *temp )
    }
 
    /* Separate XML header and Lua. */
-   start_pos = strnstr( filebuf, "<?xml ", bufsize );
-   pos       = strnstr( filebuf, "--]]", bufsize );
+   start_pos = SDL_strnstr( filebuf, "<?xml ", bufsize );
+   pos       = SDL_strnstr( filebuf, "--]]", bufsize );
    if ( pos == NULL || start_pos == NULL ) {
       WARN( _( "Event file '%s' has missing XML header!" ), file );
       return -1;
@@ -681,7 +707,7 @@ static int event_parseFile( const char *file, EventData *temp )
    }
 
    /* Check to see if syntax is valid. */
-   ret = luaL_loadbuffer( naevL, temp->lua, strlen( temp->lua ), temp->name );
+   ret = nlua_loadbuffer( naevL, temp->lua, strlen( temp->lua ), temp->name );
    if ( ret == LUA_ERRSYNTAX )
       WARN( _( "Event Lua '%s' syntax error: %s" ), file,
             lua_tostring( naevL, -1 ) );
@@ -696,9 +722,9 @@ static int event_parseFile( const char *file, EventData *temp )
 }
 
 /**
- * @brief Frees an EventData structure.
+ * @brief Frees an `EventData` structure.
  *
- *    @param event Event Data to free.
+ *    @param event `EventData` to free.
  */
 static void event_freeData( EventData *event )
 {
