@@ -20,6 +20,7 @@
 #include "array.h"
 #include "camera.h"
 #include "conf.h"
+#include "constants.h"
 #include "font.h"
 #include "gui_omsg.h"
 #include "gui_osd.h"
@@ -33,7 +34,6 @@
 #include "nlua_gfx.h"
 #include "nlua_gui.h"
 #include "nlua_tk.h"
-#include "nstring.h"
 #include "ntracing.h"
 #include "opengl.h"
 #include "pause.h"
@@ -42,7 +42,6 @@
 #include "player_gui.h"
 #include "render.h"
 #include "space.h"
-#include "spfx.h"
 #include "start.h"
 #include "toolkit.h"
 
@@ -70,17 +69,9 @@ extern unsigned int land_wid; /**< From land.c */
 /**
  * GUI Lua stuff.
  */
-static nlua_env gui_env      = LUA_NOREF; /**< Current GUI Lua environment. */
-static int      gui_L_mclick = 0;         /**< Use mouse click callback. */
-static int      gui_L_mmove  = 0;         /**< Use mouse movement callback. */
-
-/**
- * Cropping.
- */
-static double gui_viewport_x = 0.; /**< GUI Viewport X offset. */
-static double gui_viewport_y = 0.; /**< GUI Viewport Y offset. */
-static double gui_viewport_w = 0.; /**< GUI Viewport width. */
-static double gui_viewport_h = 0.; /**< GUI Viewport height. */
+static nlua_env *gui_env      = NULL; /**< Current GUI Lua environment. */
+static int       gui_L_mclick = 0;    /**< Use mouse click callback. */
+static int       gui_L_mmove  = 0;    /**< Use mouse movement callback. */
 
 /**
  * @struct Radar
@@ -103,10 +94,6 @@ typedef struct Radar_ {
    10. /**< Steps used to increase/decrease resolution. */
 static Radar gui_radar;
 
-/* needed to render properly */
-static double gui_xoff = 0.; /**< X Offset that GUI introduces. */
-static double gui_yoff = 0.; /**< Y offset that GUI introduces. */
-
 /* messages */
 static const int mesg_max = 128; /**< Maximum messages onscreen */
 static int       mesg_pointer =
@@ -131,12 +118,6 @@ static int gui_mesg_w = 0; /**< Width of messages. */
 static int gui_mesg_x = 0; /**< X positioning of messages. */
 static int gui_mesg_y = 0; /**< Y positioning of messages. */
 
-/* Calculations to speed up borders. */
-static double gui_tr = 0.; /**< Border top-right. */
-static double gui_br = 0.; /**< Border bottom-right. */
-static double gui_tl = 0.; /**< Border top-left. */
-static double gui_bl = 0.; /**< Border bottom-left. */
-
 /* Intrinsic graphical stuff. */
 static glTexture *gui_ico_hail     = NULL; /**< Hailing icon. */
 static glTexture *gui_target_spob  = NULL; /**< Spob targeting icon. */
@@ -173,12 +154,9 @@ extern void weapon_minimap( const double res, const double w, const double h,
 static void gui_renderTargetReticles( const SimpleShader *shd, double x,
                                       double y, double radius, double angle,
                                       const glColour *c );
-static void gui_borderIntersection( double *cx, double *cy, double rx,
-                                    double ry, double hw, double hh );
 /* Render GUI. */
 static void            gui_renderPilotTarget( void );
 static void            gui_renderSpobTarget( void );
-static void            gui_renderBorder( double dt );
 static void            gui_renderMessages( double dt );
 static const glColour *gui_getSpobColour( int i );
 static void gui_renderRadarOutOfRange( RadarShape sh, int w, int h, int cx,
@@ -186,12 +164,46 @@ static void gui_renderRadarOutOfRange( RadarShape sh, int w, int h, int cx,
 static void gui_blink( double cx, double cy, double vr, const glColour *col,
                        double blinkInterval, double blinkVar );
 static const glColour *gui_getPilotColour( const Pilot *p );
-static void            gui_calcBorders( void );
 /* Lua GUI. */
 static int gui_doFunc( int func_ref, const char *func_name );
 static int gui_prepFunc( int func_ref, const char *func_name );
 static int gui_runFunc( const char *func, int nargs, int nret );
 
+static double radar_linrange =
+   40.; /**< Distance (in pixels) at which it becomes linear. */
+static double radar_logscale = 40.; /**< Factor used to make a smooth transition
+                                       from linear to logarithmic. */
+void gui_logradar( double *x, double *y, double res )
+{
+   double m = hypotf( *x, *y ) / res;
+   if ( m > radar_linrange ) {
+      m        = log( m ) * radar_logscale;
+      double a = atan2( *y, *x );
+      *x       = m * cos( a );
+      *y       = m * sin( a );
+   } else {
+      *x /= res;
+      *y /= res;
+   }
+}
+
+static double unlogradar( double *x, double *y, double res )
+{
+   double m = hypotf( *x, *y );
+   if ( m > radar_linrange ) {
+      double m2 = res * exp( m / radar_logscale );
+      double a  = atan2( *y, *x );
+      *x        = m2 * cos( a );
+      *y        = m2 * sin( a );
+      // Area of "1" pixel near the location
+      return m2 - res * exp( ( m - 1. ) / radar_logscale );
+   } else {
+      *x *= res;
+      *y *= res;
+      return res;
+   }
+}
+
 /**
  * Sets the GUI to defaults.
  */
@@ -329,7 +341,7 @@ void player_messageRaw( const char *str )
       free( m->dstr );
       m->dstr = NULL;
       if ( iter.l_begin == 0 ) {
-         m->str = strndup( &str[iter.l_begin], iter.l_end - iter.l_begin );
+         m->str = SDL_strndup( &str[iter.l_begin], iter.l_end - iter.l_begin );
          gl_printRestoreInit( &m->restore );
       } else {
          m->str = malloc( iter.l_end - iter.l_begin + 2 );
@@ -348,7 +360,7 @@ void player_messageRaw( const char *str )
 /**
  * @brief Adds a mesg to the queue to be displayed on screen.
  *
- *    @param fmt String with formatting like printf.
+ *    @param fmt String with formatting like `printf`.
  */
 void player_message( const char *fmt, ... )
 {
@@ -386,7 +398,7 @@ static void gui_renderSpobTarget( void )
         ( player.p->nav_asteroid < 0 ) )
       return;
 
-      /* Make sure targets are still in range. */
+   /* Make sure targets are still in range. */
 #if 0
    if (!pilot_inRangeSpob( player.p, player.p->nav_spob )) {
       player_targetSpobSet( -1 );
@@ -401,7 +413,7 @@ static void gui_renderSpobTarget( void )
          c = &cGreen;
          x = jp->pos.x;
          y = jp->pos.y;
-         r = jumppoint_gfx->sw * 0.5;
+         r = tex_sw( jumppoint_gfx ) * 0.5;
          gui_renderTargetReticles( &shaders.targetspob, x, y, r, 0., c );
       }
    }
@@ -421,7 +433,7 @@ static void gui_renderSpobTarget( void )
 
       x = ast->sol.pos.x;
       y = ast->sol.pos.y;
-      r = ast->gfx->sw * 0.5;
+      r = tex_sw( ast->gfx ) * 0.5;
       gui_renderTargetReticles( &shaders.targetship, x, y, r, 0., c );
    }
 }
@@ -501,150 +513,12 @@ static void gui_renderPilotTarget( void )
                              c );
 }
 
-/**
- * @brief Gets the intersection with the border.
- *
- * http://en.wikipedia.org/wiki/Intercept_theorem
- *
- *    @param[out] cx X intersection.
- *    @param[out] cy Y intersection.
- *    @param rx Center X position of intersection.
- *    @param ry Center Y position of intersection.
- *    @param hw Screen half-width.
- *    @param hh Screen half-height.
- */
-static void gui_borderIntersection( double *cx, double *cy, double rx,
-                                    double ry, double hw, double hh )
-{
-   double a;
-   double w, h;
-
-   /* Get angle. */
-   a = atan2( ry, rx );
-   if ( a < 0. )
-      a += 2. * M_PI;
-
-   /* Helpers. */
-   w = hw - 7.;
-   h = hh - 7.;
-
-   /* Handle by quadrant. */
-   if ( ( a > gui_tr ) && ( a < gui_tl ) ) { /* Top. */
-      *cx = h * ( rx / ry );
-      *cy = h;
-   } else if ( ( a > gui_tl ) && ( a < gui_bl ) ) { /* Left. */
-      *cx = -w;
-      *cy = -w * ( ry / rx );
-   } else if ( ( a > gui_bl ) && ( a < gui_br ) ) { /* Bottom. */
-      *cx = -h * ( rx / ry );
-      *cy = -h;
-   } else { /* Right. */
-      *cx = w;
-      *cy = w * ( ry / rx );
-   }
-
-   /* Translate. */
-   *cx += hw;
-   *cy += hh;
-}
-
-/**
- * @brief Renders the ships/spobs in the border.
- *
- *    @param dt Current delta tick.
- */
-static void gui_renderBorder( double dt )
-{
-   (void)dt;
-   int             hw, hh;
-   double          rx, ry;
-   double          cx, cy;
-   const glColour *col;
-   Pilot *const   *pilot_stack;
-
-   NTracingZone( _ctx, 1 );
-
-   /* Get player position. */
-   hw = SCREEN_W / 2;
-   hh = SCREEN_H / 2;
-
-   /* Render borders to enhance contrast. */
-   gl_renderRect( 0., 0., 15., SCREEN_H, &cBlackHilight );
-   gl_renderRect( SCREEN_W - 15., 0., 15., SCREEN_H, &cBlackHilight );
-   gl_renderRect( 15., 0., SCREEN_W - 30., 15., &cBlackHilight );
-   gl_renderRect( 15., SCREEN_H - 15., SCREEN_W - 30., 15., &cBlackHilight );
-
-   /* Draw spobs. */
-   for ( int i = 0; i < array_size( cur_system->spobs ); i++ ) {
-      const Spob *pnt = cur_system->spobs[i];
-
-      /* Skip if unknown. */
-      if ( !spob_isKnown( pnt ) )
-         continue;
-
-      /* Check if out of range. */
-      if ( !gui_onScreenSpob( &rx, &ry, NULL, pnt ) ) {
-
-         /* Get border intersection. */
-         gui_borderIntersection( &cx, &cy, rx, ry, hw, hh );
-
-         col = gui_getSpobColour( i );
-         gl_renderCircle( cx, cy, 5, col, 0 );
-      }
-   }
-
-   /* Draw jump routes. */
-   for ( int i = 0; i < array_size( cur_system->jumps ); i++ ) {
-      const JumpPoint *jp = &cur_system->jumps[i];
-
-      /* Skip if unknown or exit-only. */
-      if ( !jp_isUsable( jp ) )
-         continue;
-
-      /* Check if out of range. */
-      if ( !gui_onScreenSpob( &rx, &ry, jp, NULL ) ) {
-
-         /* Get border intersection. */
-         gui_borderIntersection( &cx, &cy, rx, ry, hw, hh );
-
-         if ( i == player.p->nav_hyperspace )
-            col = &cGreen;
-         else
-            col = &cWhite;
-
-         gl_renderTriangleEmpty( cx, cy, -jp->angle, 10., 1., col );
-      }
-   }
-
-   /* Draw pilots. */
-   pilot_stack = pilot_getAll();
-   for ( int i = 1; i < array_size( pilot_stack ); i++ ) { /* skip the player */
-      const Pilot *plt = pilot_stack[i];
-
-      /* See if in sensor range. */
-      if ( !pilot_inRangePilot( player.p, plt, NULL ) )
-         continue;
-
-      /* Check if out of range. */
-      if ( !gui_onScreenPilot( &rx, &ry, plt ) ) {
-
-         /* Get border intersection. */
-         gui_borderIntersection( &cx, &cy, rx, ry, hw, hh );
-
-         col = gui_getPilotColour( plt );
-         gl_renderRectEmpty( cx - 5, cy - 5, 10, 10, col );
-      }
-   }
-
-   NTracingZoneEnd( _ctx );
-}
-
 /**
  * @brief Takes a pilot and returns whether it's on screen, plus its relative
  * position.
  *
- * @param[out] rx Relative X position (factoring in viewport offset)
- * @param[out] ry Relative Y position (factoring in viewport offset)
+ * @param[out] rx Relative X position.
+ * @param[out] ry Relative Y position.
  * @param pilot Pilot to determine the visibility and position of
  * @return Whether or not the pilot is on-screen.
  */
@@ -659,10 +533,6 @@ int gui_onScreenPilot( double *rx, double *ry, const Pilot *pilot )
    *rx = ( pilot->solid.pos.x - player.p->solid.pos.x ) * z;
    *ry = ( pilot->solid.pos.y - player.p->solid.pos.y ) * z;
 
-   /* Correct for offset. */
-   *rx -= gui_xoff;
-   *ry -= gui_yoff;
-
    /* Get size. */
    w = pilot->ship->size;
    h = pilot->ship->size;
@@ -681,8 +551,8 @@ int gui_onScreenPilot( double *rx, double *ry, const Pilot *pilot )
  * @brief Takes a spob or jump point and returns whether it's on screen, plus
  * its relative position.
  *
- * @param[out] rx Relative X position (factoring in viewport offset)
- * @param[out] ry Relative Y position (factoring in viewport offset)
+ * @param[out] rx Relative X position.
+ * @param[out] ry Relative Y position.
  * @param jp Jump point to determine the visibility and position of
  * @param pnt Spob to determine the visibility and position of
  * @return Whether or not the given spob is on-screen.
@@ -706,16 +576,12 @@ int gui_onScreenSpob( double *rx, double *ry, const JumpPoint *jp,
       *ry = ( jp->pos.y - player.p->solid.pos.y ) * z;
    }
 
-   /* Correct for offset. */
-   *rx -= gui_xoff;
-   *ry -= gui_yoff;
-
    /* Compare dimensions. */
    cw = SCREEN_W / 2;
    ch = SCREEN_H / 2;
    if ( tex != NULL ) {
-      cw += tex->sw / 2;
-      ch += tex->sh / 2;
+      cw += tex_sw( tex ) / 2;
+      ch += tex_sh( tex ) / 2;
    }
 
    if ( ( ABS( *rx ) > cw ) || ( ABS( *ry ) > ch ) )
@@ -725,9 +591,9 @@ int gui_onScreenSpob( double *rx, double *ry, const JumpPoint *jp,
 }
 
 /**
- * @brief Renders the gui targeting reticles.
+ * @brief Renders the GUI targeting reticles.
  *
- * @param dt Current deltatick.
+ * @param dt Current delta time.
  */
 void gui_renderReticles( double dt )
 {
@@ -763,8 +629,7 @@ void gui_render( double dt )
           ( ( player.p != NULL ) && pilot_isFlag( player.p, PILOT_DEAD ) ) ) ) {
       NTracingZone( _ctx, 1 );
 
-      gl_viewport( 0., 0., SCREEN_W, SCREEN_H );
-      spfx_cinematic();
+      // spfx_cinematic();
       gl_defViewport();
 
       NTracingZoneEnd( _ctx );
@@ -780,6 +645,7 @@ void gui_render( double dt )
       return;
 
    NTracingZone( _ctx, 1 );
+   gl_debugGroupStart();
 
    /*
     * Countdown timers.
@@ -792,14 +658,10 @@ void gui_render( double dt )
    if ( blink_spob < 0. )
       blink_spob += RADAR_BLINK_SPOB;
 
-   /* Render the border ships and targets. */
-   gui_renderBorder( dt );
-
-   /* Set viewport. */
-   gl_viewport( 0., 0., gl_screen.rw, gl_screen.rh );
+   glClear( GL_DEPTH_BUFFER_BIT );
 
    /* Run Lua. */
-   if ( gui_env != LUA_NOREF ) {
+   if ( gui_env != NULL ) {
       if ( gui_prepFunc( gui_lua_render, "render" ) == 0 ) {
          lua_pushnumber( naevL, dt );
          lua_pushnumber( naevL, dt_mod );
@@ -808,7 +670,10 @@ void gui_render( double dt )
       if ( pilot_isFlag( player.p, PILOT_COOLDOWN ) ) {
          if ( gui_prepFunc( gui_lua_render_cooldown, "render_cooldown" ) ==
               0 ) {
-            lua_pushnumber( naevL, player.p->ctimer / player.p->cdelay );
+            lua_pushnumber( naevL, CLAMP( 0., 1.,
+                                          player.p->ctimer /
+                                             ( player.p->stats.cooldown_time *
+                                               player.p->cdelay ) ) );
             lua_pushnumber( naevL, player.p->ctimer );
             gui_runFunc( "render_cooldown", 2, 0 );
          }
@@ -883,12 +748,10 @@ void gui_render( double dt )
       gl_checkErr();
    }
 
-   /* Reset viewport. */
-   gl_defViewport();
-
    /* Render messages. */
    omsg_render( dt );
 
+   gl_debugGroupEnd();
    NTracingZoneEnd( _ctx );
 }
 
@@ -912,6 +775,15 @@ int gui_radarInit( int circle, int w, int h )
    gui_radar.shape = circle ? RADAR_CIRCLE : RADAR_RECT;
    gui_radar.w     = w;
    gui_radar.h     = h;
+   if ( circle ) {
+      // In the case of the circle it's radius directly
+      radar_linrange = (double)w;
+   } else {
+      // Average width + height and compute radius
+      radar_linrange = (double)( w + h ) * 0.5 * 0.5;
+   }
+   // Make it be roughly 80% of the viewport
+   radar_linrange *= 0.8;
    gui_setRadarResolution( player.radar_res );
    return 0;
 }
@@ -949,6 +821,10 @@ void gui_radarRender( double x, double y )
    } else if ( radar->shape == RADAR_CIRCLE )
       mat4_translate_xy( &gl_view_matrix, x, y );
 
+   // Draw linear range area
+   const glColour cRange = { .r = 0.45, .g = 0.45, .b = 0.6, .a = 0.1 };
+   gl_renderCircle( 0., 0., radar_linrange, &cRange, 0 );
+
    /*
     * spobs
     */
@@ -994,9 +870,11 @@ void gui_radarRender( double x, double y )
                        radar->res, 0 );
 
    /* Render the asteroids */
+   const double render_limit =
+      radar->shape == RADAR_CIRCLE ? radar->w : INFINITY;
    for ( int i = 0; i < array_size( cur_system->asteroids ); i++ ) {
       AsteroidAnchor *ast   = &cur_system->asteroids[i];
-      double          range = EW_ASTEROID_DIST *
+      double          range = CTS.EW_ASTEROID_DIST *
                      player.p->stats.ew_detect; /* TODO don't hardcode. */
       int ax, ay, r;
       ax = round( player.p->solid.pos.x );
@@ -1006,10 +884,14 @@ void gui_radarRender( double x, double y )
                                ay + r );
       for ( int j = 0; j < il_size( &gui_qtquery ); j++ ) {
          const Asteroid *a = &ast->asteroids[il_get( &gui_qtquery, j, 0 )];
-         gui_renderAsteroid( a, radar->w, radar->h, radar->res, 0 );
+         gui_renderAsteroid( a, radar->w, radar->h, radar->res, render_limit,
+                             0 );
       }
    }
 
+   /* Render the viewport frame */
+   gui_renderViewportFrame( radar->res, render_limit, 0 );
+
    /* Render the player. */
    gui_renderPlayer( radar->res, 0 );
 
@@ -1171,7 +1053,7 @@ static const glColour *gui_getPilotColour( const Pilot *p )
  * @brief Renders a pilot in the GUI radar.
  *
  *    @param p Pilot to render.
- *    @param shape Shape of the radar (RADAR_RECT or RADAR_CIRCLE).
+ *    @param shape Shape of the radar (`RADAR_RECT` or `RADAR_CIRCLE`).
  *    @param w Width.
  *    @param h Height.
  *    @param res Radar resolution.
@@ -1193,18 +1075,19 @@ void gui_renderPilot( const Pilot *p, RadarShape shape, double w, double h,
       x = ( p->solid.pos.x / res );
       y = ( p->solid.pos.y / res );
    } else {
-      x = ( ( p->solid.pos.x - player.p->solid.pos.x ) / res );
-      y = ( ( p->solid.pos.y - player.p->solid.pos.y ) / res );
+      x = p->solid.pos.x - player.p->solid.pos.x;
+      y = p->solid.pos.y - player.p->solid.pos.y;
+      gui_logradar( &x, &y, res );
    }
    /* Get size. */
    ssize = sqrt( (double)ship_size( p->ship ) );
    scale = ( ssize + 1. ) / 2. * ( 1. + RADAR_RES_REF / res );
 
    /* Check if pilot in range. */
-   if ( ( ( shape == RADAR_RECT ) && ( ( ABS( x ) > ( w + scale ) / 2. ) ||
-                                       ( ABS( y ) > ( h + scale ) / 2. ) ) ) ||
+   if ( ( ( shape == RADAR_RECT ) && ( ( ABS( x ) > ( w + scale ) * 0.5 ) ||
+                                       ( ABS( y ) > ( h + scale ) * 0.5 ) ) ) ||
         ( ( shape == RADAR_CIRCLE ) &&
-          ( ( pow2( x ) + pow2( y ) ) > pow2( w ) ) ) ) {
+          ( ( pow2( x ) + pow2( y ) ) > pow2( w + scale ) ) ) ) {
 
       /* Draw little targeted symbol. */
       if ( p->id == player.p->target && !overlay )
@@ -1229,13 +1112,21 @@ void gui_renderPilot( const Pilot *p, RadarShape shape, double w, double h,
    scanning =
       ( pilot_isFlag( p, PILOT_SCANNING ) && ( p->target == PLAYER_ID ) );
 
-   if ( pilot_isFlag( p, PILOT_HILIGHT ) || scanning ) {
-      glColour highlighted = cRadar_hilight;
+   int hilight;
+   if ( scanning || pilot_isFlag( p, PILOT_HILIGHT ) ||
+        pilot_isFlag( p, PILOT_HAILING ) )
+      hilight = 1;
+   else
+      hilight = 0;
+
+   if ( hilight ) {
+      glColour highlighted = ( scanning ) ? cRadar_scanning : cRadar_hilight;
       highlighted.a        = 0.3;
       glUseProgram( shaders.hilight.program );
       glUniform1f( shaders.hilight.dt, animation_dt );
       gl_renderShader( x, y, scale * 2.0, scale * 2.0, 0., &shaders.hilight,
                        &highlighted, 1 );
+      hilight = 1;
    }
 
    glUseProgram( shaders.pilotmarker.program );
@@ -1248,7 +1139,7 @@ void gui_renderPilot( const Pilot *p, RadarShape shape, double w, double h,
                  RADAR_BLINK_PILOT, blink_pilot );
 
    /* Draw name. */
-   if ( overlay && pilot_isFlag( p, PILOT_HILIGHT ) ) {
+   if ( overlay && hilight ) {
       /* TODO try to minimize overlap here. */
       gl_printMarkerRaw( &gl_smallFont, x + scale + 5., y - gl_smallFont.h / 2.,
                          col, p->name );
@@ -1275,10 +1166,12 @@ void gui_renderPilot( const Pilot *p, RadarShape shape, double w, double h,
  *    @param w Width.
  *    @param h Height.
  *    @param res Radar resolution.
+ *    @param render_radius Radar radius. INFINITY if it's not a circle shape
+ * radar.
  *    @param overlay Whether to render onto the overlay.
  */
 void gui_renderAsteroid( const Asteroid *a, double w, double h, double res,
-                         int overlay )
+                         double render_radius, int overlay )
 {
    int             i, j, targeted;
    double          x, y, r, sx, sy;
@@ -1302,8 +1195,9 @@ void gui_renderAsteroid( const Asteroid *a, double w, double h, double res,
       x = ( a->sol.pos.x / res );
       y = ( a->sol.pos.y / res );
    } else {
-      x = ( ( a->sol.pos.x - player.p->solid.pos.x ) / res );
-      y = ( ( a->sol.pos.y - player.p->solid.pos.y ) / res );
+      x = a->sol.pos.x - player.p->solid.pos.x;
+      y = a->sol.pos.y - player.p->solid.pos.y;
+      gui_logradar( &x, &y, res );
    }
 
    /* Get size. */
@@ -1323,6 +1217,11 @@ void gui_renderAsteroid( const Asteroid *a, double w, double h, double res,
    /* Draw square. */
    px = MAX( x - sx, -w );
    py = MAX( y - sy, -h );
+   r  = ( sx + sy ) / 2.0 + 1.5;
+   if ( isfinite( render_radius ) &&
+        pow2( x ) + pow2( y ) > pow2( render_radius - r ) ) {
+      return;
+   }
 
    targeted =
       ( ( i == player.p->nav_asteroid ) && ( j == player.p->nav_anchor ) );
@@ -1334,7 +1233,6 @@ void gui_renderAsteroid( const Asteroid *a, double w, double h, double res,
       col = &cGrey70;
 
    // gl_renderRect( px, py, MIN( 2*sx, w-px ), MIN( 2*sy, h-py ), col );
-   r = ( sx + sy ) / 2.0 + 1.5;
    glUseProgram( shaders.asteroidmarker.program );
    gl_renderShader( px, py, r, r, 0., &shaders.asteroidmarker, col, 1 );
 
@@ -1343,6 +1241,67 @@ void gui_renderAsteroid( const Asteroid *a, double w, double h, double res,
                  blink_pilot );
 }
 
+/**
+ * @brief Renders the viewport frame in the GUI radar.
+ *
+ *    @param res Radar resolution.
+ *    @param render_radius Radar radius. INFINITY if it's not a circle shape
+ * radar.
+ *    @param overlay Whether to render onto the overlay.
+ */
+void gui_renderViewportFrame( double res, double render_radius, int overlay )
+{
+   if ( !conf.show_viewport ) {
+      return;
+   }
+
+   double vp_corner_x, vp_corner_y;
+   if ( overlay ) {
+      const double z = cam_getZoom() * res;
+      vp_corner_x    = SCREEN_W / 2.0 / z;
+      vp_corner_y    = SCREEN_H / 2.0 / z * CTS.CAMERA_VIEW_INV;
+   } else {
+      const double z = cam_getZoom();
+      // Centered on 0,0 so should be correct
+      vp_corner_x = SCREEN_W / 2.0 / z;
+      vp_corner_y = SCREEN_H / 2.0 / z * CTS.CAMERA_VIEW_INV;
+      gui_logradar( &vp_corner_x, &vp_corner_y, res );
+   }
+
+   if ( isfinite( render_radius ) &&
+        pow2( vp_corner_x ) + pow2( vp_corner_y ) > pow2( render_radius ) ) {
+      return;
+   }
+
+   double cx, cy;
+   if ( overlay ) {
+      /* overlay */
+      cam_getPos( &cx, &cy );
+      cx /= res;
+      cy /= res;
+      double ox, oy;
+      ovr_center( &ox, &oy );
+      cx += ox;
+      cy += oy;
+   } else {
+      cx = 0.0;
+      cy = 0.0;
+   }
+   const double right_angle = M_PI / 2.0;
+   glUseProgram( shaders.viewport_frame.program );
+   gl_renderShader( cx, cy - vp_corner_y, vp_corner_x, 1.0, 0.,
+                    &shaders.viewport_frame, &cRadar_viewport, 1 );
+   glUseProgram( shaders.viewport_frame.program );
+   gl_renderShader( cx + vp_corner_x, cy, vp_corner_y, 1.0, right_angle,
+                    &shaders.viewport_frame, &cRadar_viewport, 1 );
+   glUseProgram( shaders.viewport_frame.program );
+   gl_renderShader( cx, cy + vp_corner_y, vp_corner_x, 1.0, 0.,
+                    &shaders.viewport_frame, &cRadar_viewport, 1 );
+   glUseProgram( shaders.viewport_frame.program );
+   gl_renderShader( cx - vp_corner_x, cy, vp_corner_y, 1.0, right_angle,
+                    &shaders.viewport_frame, &cRadar_viewport, 1 );
+}
+
 /**
  * @brief Renders the player cross on the radar or whatever.
  */
@@ -1451,7 +1410,7 @@ static void gui_renderRadarOutOfRange( RadarShape sh, int w, int h, int cx,
 /**
  * @brief Draws the spobs in the minimap.
  *
- * Matrix mode is already displaced to center of the minimap.
+ * Matrix mode is already displaced to centre of the minimap.
  */
 void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
                      double alpha, int overlay )
@@ -1474,8 +1433,11 @@ void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
       cx = spob->pos.x / res;
       cy = spob->pos.y / res;
    } else {
-      cx = ( spob->pos.x - player.p->solid.pos.x ) / res;
-      cy = ( spob->pos.y - player.p->solid.pos.y ) / res;
+      double x = spob->pos.x - player.p->solid.pos.x;
+      double y = spob->pos.y - player.p->solid.pos.y;
+      gui_logradar( &x, &y, res );
+      cx = x;
+      cy = y;
    }
 
    /* Check if in range. */
@@ -1483,7 +1445,7 @@ void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
       GLfloat x = ABS( cx ) - r;
       GLfloat y = ABS( cy ) - r;
       /* Out of range. */
-      if ( x * x + y * y > pow2( w - 2 * r ) ) {
+      if ( x * x + y * y > pow2( w + 2. * r ) ) {
          if ( ( player.p->nav_spob == ind ) && !overlay )
             gui_renderRadarOutOfRange( RADAR_CIRCLE, w, w, cx, cy,
                                        &cRadar_tSpob );
@@ -1492,7 +1454,7 @@ void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
    } else {
       if ( shape == RADAR_RECT ) {
          /* Out of range. */
-         if ( ( ABS( cx ) - r > w / 2. ) || ( ABS( cy ) - r > h / 2. ) ) {
+         if ( ( ABS( cx ) - r > w * 0.5 ) || ( ABS( cy ) - r > h * 0.5 ) ) {
             if ( ( player.p->nav_spob == ind ) && !overlay )
                gui_renderRadarOutOfRange( RADAR_RECT, w, h, cx, cy,
                                           &cRadar_tSpob );
@@ -1532,9 +1494,12 @@ void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
 
    if ( spob->marker != NULL )
       shd = spob->marker;
-   else if ( spob_hasService( spob, SPOB_SERVICE_LAND ) )
-      shd = &shaders.spobmarker_earth;
-   else
+   else if ( spob_hasService( spob, SPOB_SERVICE_LAND ) ) {
+      if ( spob_hasService( spob, SPOB_SERVICE_INHABITED ) )
+         shd = &shaders.spobmarker_earth;
+      else
+         shd = &shaders.spobmarker_uninhabited;
+   } else
       shd = &shaders.spobmarker_empty;
 
    glUseProgram( shd->program );
@@ -1553,7 +1518,7 @@ void gui_renderSpob( int ind, RadarShape shape, double w, double h, double res,
  * @brief Renders a jump point on the minimap.
  *
  *    @param ind Jump point to render.
- *    @param shape Shape of the radar (RADAR_RECT or RADAR_CIRCLE).
+ *    @param shape Shape of the radar (`RADAR_RECT` or `RADAR_CIRCLE`).
  *    @param w Width.
  *    @param h Height.
  *    @param res Radar resolution.
@@ -1573,20 +1538,23 @@ void gui_renderJumpPoint( int ind, RadarShape shape, double w, double h,
       return;
 
    /* Default values. */
-   r  = jumppoint_gfx->sw / 2. / res;
+   r  = tex_sw( jumppoint_gfx ) / 2. / res;
    vr = overlay ? jp->mo.radius : MAX( r, 5. );
    if ( overlay ) {
       cx = jp->pos.x / res;
       cy = jp->pos.y / res;
    } else {
-      cx = ( jp->pos.x - player.p->solid.pos.x ) / res;
-      cy = ( jp->pos.y - player.p->solid.pos.y ) / res;
+      double jx = jp->pos.x - player.p->solid.pos.x;
+      double jy = jp->pos.y - player.p->solid.pos.y;
+      gui_logradar( &jx, &jy, res );
+      cx = jx;
+      cy = jy;
    }
 
    /* Check if in range. */
    if ( shape == RADAR_RECT ) {
       /* Out of range. */
-      if ( ( ABS( cx ) - r > w / 2. ) || ( ABS( cy ) - r > h / 2. ) ) {
+      if ( ( ABS( cx ) - r > w * 0.5 ) || ( ABS( cy ) - r > h * 0.5 ) ) {
          if ( ( player.p->nav_hyperspace == ind ) && !overlay )
             gui_renderRadarOutOfRange( RADAR_RECT, w, h, cx, cy,
                                        &cRadar_tSpob );
@@ -1596,7 +1564,7 @@ void gui_renderJumpPoint( int ind, RadarShape shape, double w, double h,
       x = ABS( cx ) - r;
       y = ABS( cy ) - r;
       /* Out of range. */
-      if ( x * x + y * y > pow2( w - 2 * r ) ) {
+      if ( x * x + y * y > pow2( w + 2. * r ) ) {
          if ( ( player.p->nav_hyperspace == ind ) && !overlay )
             gui_renderRadarOutOfRange( RADAR_CIRCLE, w, w, cx, cy,
                                        &cRadar_tSpob );
@@ -1650,62 +1618,6 @@ void gui_renderJumpPoint( int ind, RadarShape shape, double w, double h,
    }
 }
 
-/**
- * @brief Sets the viewport.
- */
-void gui_setViewport( double x, double y, double w, double h )
-{
-   gui_viewport_x = x;
-   gui_viewport_y = y;
-   gui_viewport_w = w;
-   gui_viewport_h = h;
-
-   /* We now set the viewport. */
-   gl_setDefViewport( gui_viewport_x, gui_viewport_y, gui_viewport_w,
-                      gui_viewport_h );
-   gl_defViewport();
-
-   /* Run border calculations. */
-   gui_calcBorders();
-}
-
-/**
- * @brief Resets the viewport.
- */
-void gui_clearViewport( void )
-{
-   gl_setDefViewport( 0., 0., gl_screen.nw, gl_screen.nh );
-   gl_defViewport();
-}
-
-/**
- * @brief Calculates and sets the GUI borders.
- */
-static void gui_calcBorders( void )
-{
-   double w, h;
-
-   /* Precalculations. */
-   w = SCREEN_W / 2.;
-   h = SCREEN_H / 2.;
-
-   /*
-    * Borders.
-    */
-   gui_tl = atan2( +h, -w );
-   if ( gui_tl < 0. )
-      gui_tl += 2 * M_PI;
-   gui_tr = atan2( +h, +w );
-   if ( gui_tr < 0. )
-      gui_tr += 2 * M_PI;
-   gui_bl = atan2( -h, -w );
-   if ( gui_bl < 0. )
-      gui_bl += 2 * M_PI;
-   gui_br = atan2( -h, +w );
-   if ( gui_br < 0. )
-      gui_br += 2 * M_PI;
-}
-
 /**
  * @brief Initializes the GUI system.
  *
@@ -1744,16 +1656,14 @@ int gui_init( void )
       vertex[15] = -3.3;
       gui_radar_select_vbo =
          gl_vboCreateStatic( sizeof( GLfloat ) * 16, vertex );
+      gl_vboLabel( gui_radar_select_vbo, "GUI Radar VBO" );
    }
 
    /* OSD */
    osd_setup( 30., SCREEN_H - 90., 150., 300. );
 
-   /* Set viewport. */
-   gui_setViewport( 0., 0., gl_screen.w, gl_screen.h );
-
    /* Icons. */
-   gui_ico_hail = gl_newSprite( GUI_GFX_PATH "hail.webp", 5, 2, 0 );
+   gui_ico_hail = gl_newSprite( GUI_GFX_PATH "hail", 5, 2, 0 );
 
    /* Quadtrees. */
    il_create( &gui_qtquery, 1 );
@@ -1764,14 +1674,14 @@ int gui_init( void )
 /**
  * @brief Runs a GUI Lua function.
  *
- *    @param func_ref Reference of the functionn to run.
+ *    @param func_ref Reference of the function to run.
  *    @param func_name Name of the function to run.
  *    @return 0 on success.
  */
 static int gui_doFunc( int func_ref, const char *func_name )
 {
    int ret;
-   if ( gui_env == LUA_NOREF )
+   if ( gui_env == NULL )
       return -1;
 
    ret = gui_prepFunc( func_ref, func_name );
@@ -1783,7 +1693,7 @@ static int gui_doFunc( int func_ref, const char *func_name )
 /**
  * @brief Prepares to run a function.
  *
- *    @param func_ref Reference of the functionn to prepare.
+ *    @param func_ref Reference of the function to prepare.
  *    @param func_name Name of the function to prepare.
  *    @return 0 on success.
  */
@@ -1791,7 +1701,7 @@ static int gui_prepFunc( int func_ref, const char *func_name )
 {
    (void)func_name;
 #if DEBUGGING
-   if ( gui_env == LUA_NOREF ) {
+   if ( gui_env == NULL ) {
       WARN( _( "GUI '%s': Trying to run GUI func '%s' but no GUI is loaded!" ),
             gui_name, func_name );
       return -1;
@@ -1844,7 +1754,7 @@ static int gui_runFunc( const char *func, int nargs, int nret )
  */
 void gui_reload( void )
 {
-   if ( gui_env == LUA_NOREF )
+   if ( gui_env == NULL )
       return;
 
    gui_load( gui_pick() );
@@ -1859,7 +1769,7 @@ void gui_setCargo( void )
 }
 
 /**
- * @brief Player just changed their nav computer target.
+ * @brief Player just changed their navigation computer target.
  */
 void gui_setNav( void )
 {
@@ -1912,7 +1822,7 @@ void gui_updateEffects( void )
  */
 void gui_setGeneric( const Pilot *pilot )
 {
-   if ( gui_env == LUA_NOREF )
+   if ( gui_env == NULL )
       return;
 
    if ( player_isFlag( PLAYER_DESTROYED ) || player_isFlag( PLAYER_CREATING ) ||
@@ -1983,7 +1893,7 @@ int gui_load( const char *name )
 
    /* Clean up. */
    nlua_freeEnv( gui_env );
-   gui_env = LUA_NOREF;
+   gui_env = NULL;
 
    /* Create Lua state. */
    gui_env = nlua_newEnv( name );
@@ -1999,7 +1909,7 @@ int gui_load( const char *name )
                "Most likely Lua file has improper syntax, please check" ),
             path, lua_tostring( naevL, -1 ) );
       nlua_freeEnv( gui_env );
-      gui_env = LUA_NOREF;
+      gui_env = NULL;
       free( buf );
       return -1;
    }
@@ -2026,13 +1936,7 @@ int gui_load( const char *name )
    /* Run create function. */
    if ( gui_doFunc( gui_lua_create, "create" ) ) {
       nlua_freeEnv( gui_env );
-      gui_env = LUA_NOREF;
-   }
-
-   /* Recreate land window if landed. */
-   if ( landed && land_spob ) {
-      land_genWindows( 0 );
-      window_lower( land_wid );
+      gui_env = NULL;
    }
 
    /* Add the GUI to the player. */
@@ -2053,22 +1957,15 @@ void gui_cleanup( void )
    gui_mouseClickEnable( 0 );
    gui_mouseMoveEnable( 0 );
 
-   /* Set the viewport. */
-   gui_clearViewport();
-
    /* Set overlay bounds. */
    ovr_boundsSet( 0, 0, 0, 0 );
 
    /* Reset FPS. */
    fps_setPos( 15., (double)( gl_screen.h - 15 - gl_defFontMono.h ) );
 
-   /* Destroy offset. */
-   gui_xoff = 0.;
-   gui_yoff = 0.;
-
    /* Destroy lua. */
    nlua_freeEnv( gui_env );
-   gui_env = LUA_NOREF;
+   gui_env = NULL;
 
    /* OMSG */
    omsg_position( SCREEN_W / 2., SCREEN_H * 2. / 3., SCREEN_W * 2. / 3. );
@@ -2102,7 +1999,7 @@ void gui_cleanup( void )
 }
 
 /**
- * @brief Frees the gui stuff.
+ * @brief Frees the GUI stuff.
  */
 void gui_free( void )
 {
@@ -2136,6 +2033,9 @@ void gui_free( void )
 void gui_setRadarResolution( double res )
 {
    gui_radar.res = CLAMP( RADAR_RES_MIN, RADAR_RES_MAX, res );
+
+   // Scalings for the radar
+   radar_logscale = radar_linrange / log( radar_linrange );
 }
 
 /**
@@ -2150,18 +2050,6 @@ void gui_setRadarRel( int mod )
    player_message( _( "#oRadar set to %.0fx.#0" ), round( gui_radar.res ) );
 }
 
-/**
- * @brief Gets the GUI offset.
- *
- *    @param x X offset.
- *    @param y Y offset.
- */
-void gui_getOffset( double *x, double *y )
-{
-   *x = gui_xoff;
-   *y = gui_yoff;
-}
-
 /**
  * @brief Gets the hail icon texture.
  */
@@ -2191,7 +2079,7 @@ void gui_targetPilotGFX( const glTexture *gfx )
 /**
  * @brief Translates a mouse position from an SDL_Event to GUI coordinates.
  */
-static void gui_eventToScreenPos( int *sx, int *sy, int ex, int ey )
+static void gui_eventToScreenPos( float *sx, float *sy, float ex, float ey )
 {
    gl_windowToScreenPos( sx, sy, ex, ey );
 }
@@ -2204,7 +2092,8 @@ static void gui_eventToScreenPos( int *sx, int *sy, int ex, int ey )
  */
 int gui_radarClickEvent( SDL_Event *event )
 {
-   int    mxr, myr, in_bounds;
+   float  mxr, myr;
+   int    in_bounds;
    double x, y, cx, cy;
 
    gui_eventToScreenPos( &mxr, &myr, event->button.x, event->button.y );
@@ -2221,72 +2110,13 @@ int gui_radarClickEvent( SDL_Event *event )
    }
    if ( !in_bounds )
       return 0;
-   x = ( mxr - cx ) * gui_radar.res + player.p->solid.pos.x;
-   y = ( myr - cy ) * gui_radar.res + player.p->solid.pos.y;
-   return input_clickPos( event, x, y, 1., 10. * gui_radar.res,
-                          15. * gui_radar.res );
-}
-
-/**
- * @brief Handles clicks on the GUI border icons.
- *
- *    @brief event The click event.
- *    @return Whether the click was used to trigger an action.
- */
-int gui_borderClickEvent( SDL_Event *event )
-{
-   unsigned int pid;
-   double       ang, angp, mouseang;
-   int          mx, my;
-   int          pntid, jpid, astid, fieid;
-   double       x, y;
-   int          autonav = ( event->button.button == SDL_BUTTON_RIGHT ) ? 1 : 0;
-   double       px      = player.p->solid.pos.x;
-   double       py      = player.p->solid.pos.y;
-   gui_eventToScreenPos( &mx, &my, event->button.x, event->button.y );
-   mx -= gui_viewport_x;
-   my -= gui_viewport_y;
-
-   /* No intersection with border. */
-   if ( !( ( mx <= 15 || my <= 15 ) ||
-           ( my >= gl_screen.h - 15 || mx >= gl_screen.w - 15 ) ) )
-      return 0;
-
-   /* Border targeting is handled as a special case, as it uses angles,
-    * not coordinates. */
-   x        = ( mx - ( gl_screen.w / 2. ) ) + px;
-   y        = ( my - ( gl_screen.h / 2. ) ) + py;
-   mouseang = atan2( py - y, px - x );
-   angp     = pilot_getNearestAng( player.p, &pid, mouseang, 1 );
-   ang = system_getClosestAng( cur_system, &pntid, &jpid, &astid, &fieid, x, y,
-                               mouseang );
-
-   if ( ( ABS( angle_diff( mouseang, angp ) ) > M_PI / 64 ) ||
-        ABS( angle_diff( mouseang, ang ) ) <
-           ABS( angle_diff( mouseang, angp ) ) )
-      pid = PLAYER_ID; /* Pilot angle is too great, or spob/jump is closer. */
-   if ( ABS( angle_diff( mouseang, ang ) ) > M_PI / 64 )
-      jpid = pntid = astid = fieid =
-         -1; /* Spob angle difference is too great. */
-
-   if ( pid != PLAYER_ID ) {
-      if ( input_clickedPilot( pid, autonav ) )
-         return 1;
-   } else if ( pntid >= 0 ) { /* Spob is closest. */
-      if ( input_clickedSpob( pntid, autonav, 1 ) )
-         return 1;
-   } else if ( jpid >= 0 ) { /* Jump point is closest. */
-      if ( input_clickedJump( jpid, autonav ) )
-         return 1;
-   } else if ( astid >= 0 ) { /* Asteroid is closest. */
-      if ( input_clickedAsteroid( fieid, astid ) )
-         return 1;
-   } else if ( pntid >= 0 ) { /* Spob is closest. */
-      if ( input_clickedSpob( pntid, autonav, 0 ) )
-         return 1;
-   }
-
-   return 0;
+   x                  = mxr - cx;
+   y                  = myr - cy;
+   double sensitivity = unlogradar( &x, &y, gui_radar.res );
+   x += player.p->solid.pos.x;
+   y += player.p->solid.pos.y;
+   return input_clickPos( event, x, y, 1., 10. * sensitivity,
+                          15. * sensitivity );
 }
 
 /**
@@ -2294,12 +2124,12 @@ int gui_borderClickEvent( SDL_Event *event )
  */
 int gui_handleEvent( SDL_Event *evt )
 {
-   int ret;
-   int x, y;
+   int   ret;
+   float x, y;
 
    if ( player.p == NULL )
       return 0;
-   if ( ( evt->type == SDL_MOUSEBUTTONDOWN ) &&
+   if ( ( evt->type == SDL_EVENT_MOUSE_BUTTON_DOWN ) &&
         ( pilot_isFlag( player.p, PILOT_HYP_PREP ) ||
           pilot_isFlag( player.p, PILOT_HYP_BEGIN ) ||
           pilot_isFlag( player.p, PILOT_HYPERSPACE ) ) )
@@ -2308,7 +2138,7 @@ int gui_handleEvent( SDL_Event *evt )
    ret = 0;
    switch ( evt->type ) {
    /* Mouse motion. */
-   case SDL_MOUSEMOTION:
+   case SDL_EVENT_MOUSE_MOTION:
       if ( !gui_L_mmove )
          break;
       if ( gui_prepFunc( gui_lua_mouse_move, "mouse_move" ) == 0 ) {
@@ -2320,8 +2150,8 @@ int gui_handleEvent( SDL_Event *evt )
       break;
 
    /* Mouse click. */
-   case SDL_MOUSEBUTTONDOWN:
-   case SDL_MOUSEBUTTONUP:
+   case SDL_EVENT_MOUSE_BUTTON_DOWN:
+   case SDL_EVENT_MOUSE_BUTTON_UP:
       if ( !gui_L_mclick )
          break;
       if ( gui_prepFunc( gui_lua_mouse_click, "mouse_click" ) == 0 ) {
@@ -2329,7 +2159,7 @@ int gui_handleEvent( SDL_Event *evt )
          gui_eventToScreenPos( &x, &y, evt->button.x, evt->button.y );
          lua_pushnumber( naevL, x );
          lua_pushnumber( naevL, y );
-         lua_pushboolean( naevL, ( evt->type == SDL_MOUSEBUTTONDOWN ) );
+         lua_pushboolean( naevL, ( evt->type == SDL_EVENT_MOUSE_BUTTON_DOWN ) );
          gui_runFunc( "mouse_click", 4, 1 );
          ret = lua_toboolean( naevL, -1 );
          lua_pop( naevL, 1 );
