@@ -4,7 +4,7 @@
 #pragma once
 
 /** @cond */
-#include "SDL_events.h"
+#include <SDL3/SDL_events.h>
 /** @endcond */
 
 #define NMOD_NONE 0
@@ -112,7 +112,6 @@ typedef enum {
  * set input
  */
 void        input_setDefault( int wasd );
-SDL_Keycode input_keyConv( const char *name );
 void        input_setKeybind( KeySemanticType keybind, KeybindType type,
                               SDL_Keycode key, SDL_Keymod mod );
 const char *input_modToText( SDL_Keymod mod );
@@ -134,10 +133,6 @@ void       input_disableAll( void );
 void       input_toggleEnable( KeySemanticType key, int enable );
 int        input_clickPos( SDL_Event *event, double x, double y, double zoom,
                            double minpr, double minr );
-int        input_clickedJump( int jump, int autonav );
-int        input_clickedSpob( int spob, int autonav, int priority );
-int        input_clickedAsteroid( int field, int asteroid );
-int        input_clickedPilot( unsigned int pilot, int autonav );
 void       input_clicked( const void *clicked );
 int        input_isDoubleClick( const void *clicked );
 
@@ -163,3 +158,7 @@ void input_update( double dt );
 void input_mouseShow( void );
 void input_mouseHide( void );
 int  input_mouseIsShown( void );
+
+// Exported from Rust
+const char *input_keyToStr( SDL_Keycode key );
+SDL_Keycode input_keyFromStr( const char *name );
